-- ========================================================================== --
-- 										 EskaQuestTracker                                       --
-- @Author   : Skamer <https://mods.curse.com/members/DevSkamer>              --
-- @Website  : https://wow.curseforge.com/projects/eska-quest-tracker         --
-- ========================================================================== --
Scorpio                "EskaQuestTracker.Classes.Objective"              "1.0.0"
-- ========================================================================== --
namespace "EQT"                                                               --
import "System"                                                               --
-- ========================================================================== --
-- Set the default values for the DB
function OnLoad(self)

	_DB:SetDefault("Objective", {
		colors = {
			completed = { r = 0, g = 1, b = 0},
			inProgress = { r = 148/255, g = 148/255, b = 148/255 }

		}
	})
end

class "Objective" extend "IReusable" "IFrame"

	_ObjectiveCache = setmetatable( {}, { __mode = "k" } )


	local function CreateStatusBar(self)
		local bar = CreateFrame("StatusBar", nil, self.frame)
		bar:SetStatusBarTexture([[Interface\AddOns\EskaQuestTracker\Media\Textures\Frame-Background-6]])
		--bar:SetStatusBarColor(1, 216/255, 0, 0.7)
		bar:SetStatusBarColor(0, 148/255, 1, 0.6)
		bar:SetMinMaxValues(0, 1)
		bar:SetValue(0.6)

		local text = bar:CreateFontString(nil, "OVERLAY", GameFontHighlightSmall)
		local color = { r = 0, g = 148 / 255, b = 255 / 255 }
		local font = _LibSharedMedia:Fetch("font", "PT Sans Bold Italic")

		text:SetTextColor(1, 1, 1, 1)
		text:SetAllPoints()
		text:SetFont(font, 9)
		text:SetJustifyH("CENTER")
		text:SetJustifyV("MIDDLE")
		bar.text = text


		local bgFrame = CreateFrame("Frame", nil, bar)
		--bgFrame:InsetPoints(bar, -2, -2)
		bgFrame:SetPoint("TOPLEFT", -2, 2)
		bgFrame:SetPoint("BOTTOMRIGHT", 2, -2)
		bgFrame:SetFrameLevel(bgFrame:GetFrameLevel() - 1)

		bgFrame.background = bgFrame:CreateTexture(nil, "BACKGROUND")
		bgFrame.background:SetAllPoints(bgFrame)
		bgFrame.background:SetTexture([[Interface\AddOns\EskaQuestTracker\Media\Textures\Frame-Background-6]])
		bgFrame.background:SetVertexColor(0, 0, 0, 0.5)

		local borderB = bgFrame:CreateTexture(nil,"OVERLAY")
		borderB:SetColorTexture(0,0,0)
		borderB:SetPoint("BOTTOMLEFT")
		borderB:SetPoint("BOTTOMRIGHT")
		borderB:SetHeight(2)

		local borderT = bgFrame:CreateTexture(nil,"OVERLAY")
		borderT:SetColorTexture(0,0,0)
		borderT:SetPoint("TOPLEFT")
		borderT:SetPoint("TOPRIGHT")
		borderT:SetHeight(2)

		local borderL = bgFrame:CreateTexture(nil,"OVERLAY")
		borderL:SetColorTexture(0,0,0)
		borderL:SetPoint("TOPLEFT")
		borderL:SetPoint("BOTTOMLEFT")
		borderL:SetWidth(2)

		local borderR = bgFrame:CreateTexture(nil,"OVERLAY")
		borderR:SetColorTexture(0,0,0)
		borderR:SetPoint("TOPRIGHT")
		borderR:SetPoint("BOTTOMRIGHT")
		borderR:SetWidth(2)


		return bar
	end
	-- ======================================================================== --
	-- Methods                                                                  --
	-- ======================================================================== --
  function SetQuest(self, quest)
    self.quest = quest
  end

	__Arguments__{}
  function ShowProgress(self)
		if not self.frame.fbar then
			local fbar = CreateStatusBar(self)
			fbar:SetHeight(12)
			fbar:SetPoint("TOPLEFT", self.frame.text, "BOTTOMLEFT", 0, -4)
			fbar:SetPoint("TOPRIGHT", self.frame.text, "BOTTOMRIGHT", -35, 0) -- 25
			self.frame.fbar = fbar
		end

		self.frame.fbar:Show()


		self.height = self.baseHeight + 16
  end

  function HideProgress(self)
		if self.frame.fBar then
			self.frame.fbar:Hide()
		end
		self.height = self.baseHeight
  end


	__Arguments__{ String }
	function SetTextProgress(self, text)
		if self.frame.fbar then
			self.frame.fbar.text:SetText(text)
		end
	end

	__Arguments__{ Number }
	function SetProgress(self, progress)
		if self.frame.fbar then
			self.frame.fbar:SetValue(progress)
		end
	end

	__Arguments__{ Number, Number }
	function SetMinMaxProgress(self, min, max)
		if self.frame.fbar then
			self.frame.fbar:SetMinMaxValues(min, max)
		end
	end

	function Refresh(self, ...)
		local color
		if self.isCompleted then
			color = Objective.fontColorCompleted
		else
			color = Objective.fontColorInProgress
		end
		self.frame.square:SetBackdropColor(color.r, color.g, color.b)
		self.frame.text:SetTextColor(color.r, color.g, color.b)
	end

	__Static__() function RefreshAll()
		for obj in pairs(_ObjectiveCache) do
			obj:Refresh()
		end
	end

	function Reset(self)
		self.text = nil
		self.type = nil
		self.isCompleted = nil

		self:ClearAllPoints()
		self:SetParent(nil)
		self:Hide()
		self:HideProgress()

		self.height = 16
	end

	-- ======================================================================== --
	-- Handlers
	-- ======================================================================== --
	local function SetText(self, new, old, prop)
		self.frame.text:SetText(new)
	end

	local function SetCompleted(self, new, old, prop)
		if new then
			self.frame.square:SetBackdropColor(0, 1, 0)
			self.frame.text:SetTextColor(0, 1, 0)
		else
			self.frame.square:SetBackdropColor(0.5, 0.5, 0.5)
			self.frame.text:SetTextColor(148/255, 148/255, 148/255)
		end
	end
	-- ======================================================================== --
	-- Properties
	-- ======================================================================== --
	property "text" { TYPE = String, DEFAULT = "", HANDLER = SetText }
	property "type" { TYPE = String, DEFAULT = "" }
	property "isCompleted" { TYPE = Boolean, DEFAULT = false, HANDLER = SetCompleted}

	--property "text" { TYPE = String, DEFAULT = "", FIELD = "__text", Set="SetText", Get="GetText"}
	--property "type" { TYPE = String }
	--property "isCompleted" { TYPE = String, DEFAULT = false, Set="SetCompleted", Get="IsCompleted"}

	-- the font color used when the objective has been completed
	__Static__() property "fontColorCompleted" {
		Set = function(self, color)
			_DB.Objective.colors.completed = color ; Objective:RefreshAll()
		end,
		Get = function(self) return _DB.Objective.colors.completed end,
	}
	-- the font color used when the objective is in progress
	__Static__() property "fontColorInProgress" {
		Set = function(self, color)
			_DB.Objective.colors.inProgress = color ; Objective:RefreshAll()
		end,
		Get = function(self) return _DB.Objective.colors.inProgress end,
	}

	-- ======================================================================== --
	-- Contructor
	-- ======================================================================== --
  function Objective(self)

    local frame = CreateFrame("Frame")
    frame:SetBackdrop(_Backdrops.Common)
    frame:SetBackdropColor(0.1, 0.1, 0.1, 0)
    frame:SetBackdropBorderColor(0, 0, 0, 0)
		--self:SetFrame(frame)
		--self:SetHeight(12)

    local square = CreateFrame("Frame", nil, frame)
    square:SetBackdrop(_Backdrops.Common)
    square:SetWidth(8)
    square:SetHeight(8)
    square:SetPoint("TOPLEFT", 10, -4)
    square:SetBackdropBorderColor(0, 0, 0)
    frame.square = square

    local text = frame:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
		local font = _LibSharedMedia:Fetch("font", "PT Sans Narrow Bold")

    text:SetPoint("TOPLEFT", square, "TOPRIGHT")
    text:SetPoint("TOPRIGHT")
    text:SetFont(font, 9, "OUTLINE") -- _Addon.Fonts.Test
    text:SetJustifyH("LEFT")
    text:SetHeight(10)

    frame.text = text

    --self:SetFrame(frame)
    --self:SetCompleted(false)
		self.frame = frame
		--self.isCompleted = false
		self.height = 16
		self.baseHeight = self.height

		self:Refresh()

		_ObjectiveCache[self] = true
  end

endclass "Objective"
