-- ========================================================================== --
-- 										 EskaQuestTracker                                       --
-- @Author   : Skamer <https://mods.curse.com/members/DevSkamer>              --
-- @Website  : https://wow.curseforge.com/projects/eska-quest-tracker         --
-- ========================================================================== --
Scorpio                   "EskaQuestTracker.Quests"                      "1.0.0"
-- ========================================================================== --
namespace "EQT"
-- ========================================================================== --
GetNumQuestLogEntries      = GetNumQuestLogEntries
GetQuestLogTitle           = GetQuestLogTitle
GetNumQuestLogEntries      = GetNumQuestLogEntries
GetQuestLogTitle           = GetQuestLogTitle
GetQuestLogIndexByID       = GetQuestLogIndexByID
GetQuestWatchIndex         = GetQuestWatchIndex
GetQuestLogSpecialItemInfo = GetQuestLogSpecialItemInfo
GetQuestObjectiveInfo      = GetQuestObjectiveInfo
GetDistanceSqToQuest       = GetDistanceSqToQuest
AddQuestWatch              = AddQuestWatch
SelectQuestLogEntry        = SelectQuestLogEntry
IsWorldQuest               = QuestUtils_IsQuestWorldQuest
IsQuestBounty              = IsQuestBounty
-- IsQuestTask
-- ======================[[ DATA ]]========================================== --
QUESTS_CACHE               = {}
QUEST_HEADERS_CACHE        = {}
QUEST_LOCATIONS            = {}
-- ========================================================================== --
function OnLoad(self)
  _DB:SetDefault("Quests", {
    sortByDistance = true,
    filteringByZone = false,
  })

end

__Thread__()
function OnEnable(self)
  if not _QuestBlock then
    _QuestBlock = QuestBlock()
    _Addon:RegisterBlock(_QuestBlock)
     Wait("QUEST_LOG_UPDATE")
  end

  self:LoadQuests()
  self:UpdateQuestHeadersCache()
  self:UpdateDistance()
end


function OnDisable(self)
  if _QuestBlock then
    _QuestBlock.isActive = false
  end

end

__SystemEvent__()
function PLAYER_ALIVE()
  _M:LoadQuests()
end

__Thread__()
__SystemEvent__()
function QUEST_ACCEPTED(index, questID)
  -- Don't continue if the quest is a world quest
  if IsWorldQuest(questID) or IsQuestBounty(questID) then return end

  Debug("Accept  quest id :  %i", questID)

  -- Get a quest object
  local quest = _ObjectManager:GetQuest()
  quest.id = questID
  quest.header = _M:GetQuestHeader(questID)

  -- Add in the cache
  QUESTS_CACHE[quest.id] = quest

  -- @HACK : Set a little delay to get a valid quest item
  Delay(0.1)
  -- Update the quest and get its objectives
  _M:UpdateQuest(quest)

  -- Add it in the block
  _QuestBlock:AddQuest(quest)
  -- Refresh the block displaying
  _QuestBlock:Draw()
end

__SystemEvent__()
function QUEST_REMOVED(questID)
  _QuestBlock:RemoveQuest(questID)
  _QuestBlock:Draw()
end


function LoadQuests(self)
  local numEntries, numQuests = GetNumQuestLogEntries()
  local currentHeader = "MISC"

  for i = 1, numEntries do
    local title, level, suggestedGroup, isHeader, isCollapsed, isComplete,
    frequency, questID, startEvent, displayQuestID, isOnMap, hasLocalPOI,
    isTask, isBounty, isStory, isHidden = GetQuestLogTitle(i)

    if not isTask and not QUESTS_CACHE[questID] then
      if isHeader then
        currentHeader = title
      elseif not isHeader and not isHidden then
        AddQuestWatch(i)
        local quest = _ObjectManager:GetQuest()
        quest.id = questID
        quest.name = title
        quest.level = level
        quest.isHidden = isHidden
        quest.header = currentHeader

        QUESTS_CACHE[questID] = quest
        self:UpdateQuest(quest)

        _QuestBlock:AddQuest(quest)
      end
    end
  end
  _QuestBlock:Draw()
end

function UpdateQuestHeadersCache()
  wipe(QUEST_HEADERS_CACHE)
  local currentHeader = "Misc"
  local numEntries, numQuests = GetNumQuestLogEntries()

  for i = 1, numEntries do
      local title, _, _, isHeader, _, _, _, questID = GetQuestLogTitle(i)
      if isHeader then
        currentHeader = title
      else
        QUEST_HEADERS_CACHE[questID] = currentHeader
      end
  end
end

function GetQuestHeader(self, qID)
  -- Check if the quest header is in the cache
  if QUEST_HEADERS_CACHE[qID] then
    -- if yes, return the header in the cache
    return QUEST_HEADERS_CACHE[qID]
  end

  -- if no, find the quest header
  local currentHeader = "Misc"
  local numEntries, numQuests = GetNumQuestLogEntries()

  for i = 1, numEntries do
    local title, _, _, isHeader, _, _, _, questID = GetQuestLogTitle(i)
    if isHeader then
      currentHeader = title
    elseif questID == qID then
      QUEST_HEADERS_CACHE[qID] = currentHeader
      return currentHeader
    end
  end

  return currentHeader
end

function UpdateQuest(self, quest)
  local questLogIndex = GetQuestLogIndexByID(quest.id)
  local questWatchIndex = GetQuestWatchIndex(questLogIndex)

  local distanceSq, onContinent = GetDistanceSqToQuest(questLogIndex)

  if not questWatchIndex then return end

    local questID, title, questLogIndex, numObjectives, requiredMoney,
    isComplete, startEvent, isAutoComplete, failureTime, timeElapsed,
    questType, isTask, isBounty, isStory, isOnMap, hasLocalPOI = GetQuestWatchInfo(questWatchIndex)

    local itemLink, itemTexture = GetQuestLogSpecialItemInfo(questLogIndex)

    quest.name = title .. string.format(" [%d]", quest.id)
    quest.isOnMap = isOnMap
    quest.isTask = isTask
    quest.isBounty = isBounty
    quest.distance = distanceSq

    if itemLink and itemTexture then
      local itemQuest = quest:GetQuestItem()

      itemQuest.link = itemLink
      itemQuest.texture = itemTexture
    end

    if numObjectives > 0 then
      quest.numObjectives = numObjectives
      for index = 1, numObjectives do
        local text, type, finished = GetQuestObjectiveInfo(quest.id, index, false)
        local objective = quest:GetObjective(index)

        objective.isCompleted = finished
        objective.text = text
      end
    else
      quest.numObjectives = 1
      local objective = quest:GetObjective(1)
      SelectQuestLogEntry(questLogIndex)
      objective.text = GetQuestLogCompletionText()
      objective.isCompleted = false
    end
end

__SystemEvent__ "PLAYER_ENTERING_WORLD" "QUEST_LOG_UPDATE" "QUEST_WATCH_LIST_CHANGED" "QUEST_POI_UPDATE"
"ZONE_CHANGED_NEW_AREA" "ZONE_CHANGED"
function QUESTS_UPDATE()
  for _, quest in pairs(QUESTS_CACHE) do
    _M:UpdateQuest(quest)
  end
end


__Thread__()
function UpdateDistance()
  while QuestBlock.sortByDistance do
    for _, quest in pairs(QUESTS_CACHE) do
      local questLogIndex = GetQuestLogIndexByID(quest.id)
      local distanceSq, onContinent = GetDistanceSqToQuest(questLogIndex)

      quest.distance = math.sqrt(distanceSq)
    end
    _QuestBlock:Draw()
    Next()
  end
end
